@@ -1,6 +1,7 @@
 """
 API for easily writing an ODC Dataset
 """
+
 import shutil
 import tempfile
 import uuid
@@ -9,7 +10,18 @@ from copy import deepcopy
 from enum import Enum, auto
 from pathlib import Path, PosixPath, PurePath
 from textwrap import dedent
-from typing import Any, Dict, Generator, Iterable, List, Optional, Tuple, Union
+from typing import (
+    Any,
+    ClassVar,
+    Dict,
+    Generator,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Union,
+)
 from urllib.parse import urlsplit
 
 import numpy
@@ -146,7 +158,7 @@ class DatasetPrepare(Eo3Interface):
     #:
     #: These are fields that are inherent to the underlying observation, and so will
     #: still be relevant after most 1:1 processing.
-    INHERITABLE_PROPERTIES = {
+    INHERITABLE_PROPERTIES: ClassVar[Set[str]] = {
         "datetime",
         "dtr:end_datetime",
         "dtr:start_datetime",
@@ -452,8 +464,7 @@ class DatasetPrepare(Eo3Interface):
     def __enter__(self) -> "DatasetPrepare":
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        ...
+    def __exit__(self, exc_type, exc_val, exc_tb): ...
 
     @property
     def collection_location(self) -> Path:
@@ -670,7 +681,6 @@ class DatasetPrepare(Eo3Interface):
         source_dataset: DatasetDoc,
         inherit_skip_properties: Optional[List[str]] = None,
     ):
-
         if not inherit_skip_properties:
             # change the inherit_skip_properties to [] if it is None. Make the 'in list check' easier.
             inherit_skip_properties = []
@@ -1355,6 +1365,9 @@ class DatasetAssembler(DatasetPrepare):
         and X/Y or lat/long dimensions and coordinates. These are used to
         create an ODC GeoBox.
 
+        Nodata values will be taken from the ".nodata" attribute on each
+        individual array if it exists and no custom nodata value is provided.
+
         :param xarray.Dataset dataset: an xarray dataset (as returned by
                 :meth:`datacube.Datacube.load` and other methods)
 
@@ -1363,6 +1376,13 @@ class DatasetAssembler(DatasetPrepare):
         grid_spec = images.GridSpec.from_odc_xarray(dataset)
         for name, dataarray in dataset.data_vars.items():
             name: str
+
+            # Get nodata attribute from array if nodata is not provided
+            # and attribute is not None
+            nodata_value = (
+                dataarray.attrs.get("nodata", None) if nodata is None else nodata
+            )
+
             self._write_measurement(
                 name,
                 dataarray.data,
@@ -1374,7 +1394,7 @@ class DatasetAssembler(DatasetPrepare):
                 expand_valid_data=expand_valid_data,
                 overview_resampling=overview_resampling,
                 overviews=overviews,
-                nodata=nodata,
+                nodata=nodata_value,
             )
 
     def _write_measurement(
