@@ -7,6 +7,7 @@ from collections import defaultdict
 from enum import Enum, auto
 from pathlib import Path, PurePath
 from typing import (
+    ClassVar,
     Dict,
     Generator,
     Iterable,
@@ -146,7 +147,7 @@ class GridSpec:
         return cls(
             shape=shape,
             transform=dataset.geobox.transform,
-            crs=CRS.from_wkt(dataset.geobox.crs.crs_str),
+            crs=CRS.from_wkt(str(dataset.geobox.crs)),
         )
 
     @classmethod
@@ -429,14 +430,16 @@ class MeasurementBundler:
         grid_docs: Dict[str, GridDoc] = {}
         measurement_docs: Dict[str, MeasurementDoc] = {}
         crs = None
+
         for grid_name, (grid, measurements) in self._as_named_grids().items():
             # Validate assumption: All grids should have same CRS
             if crs is None:
                 crs = grid.crs
+
             # TODO: CRS equality is tricky. This may not work.
             #       We're assuming a group of measurements specify their CRS
             #       the same way if they are the same.
-            elif grid.crs != crs:
+            elif (grid.crs is not None) and grid.crs != crs:
                 raise ValueError(
                     f"Measurements have different CRSes in the same dataset:\n"
                     f"\t{crs.to_string()!r}\n"
@@ -563,7 +566,7 @@ class FileWrite:
     This code is derived from the old eugl packaging code and can probably be improved.
     """
 
-    PREDICTOR_DEFAULTS = {
+    PREDICTOR_DEFAULTS: ClassVar[Dict[str, int]] = {
         "int8": 2,
         "uint8": 2,
         "int16": 2,
@@ -707,6 +710,10 @@ class FileWrite:
             "driver": "GTiff",
             "predictor": self.PREDICTOR_DEFAULTS[dtype],
         }
+        # Ensure 'nan' is always tagged as nodata for floating point types
+        if nodata is None and numpy.issubdtype(array.dtype, numpy.floating):
+            nodata = numpy.nan
+
         if nodata is not None:
             rio_args["nodata"] = nodata
 
@@ -901,7 +908,11 @@ class FileWrite:
         out_crs = ql_grid.crs
 
         # Scale and write as JPEG to the output.
-        (thumb_transform, thumb_width, thumb_height,) = calculate_default_transform(
+        (
+            thumb_transform,
+            thumb_width,
+            thumb_height,
+        ) = calculate_default_transform(
             out_crs,
             out_crs,
             ql_grid.shape[1],
