@@ -155,6 +155,10 @@ LANDSAT_xTM_BAND_ALIASES = {
     "emissivity": "emissivity",
     "emissivity_stdev": "emissivity_stdev",
     "cloud_distance": "cloud_distance",
+    "angle_sensor_azimuth_band_4": "view_azimuth",
+    "angle_sensor_zenith_band_4": "view_zenith",
+    "angle_solar_azimuth_band_4": "solar_azimuth",
+    "angle_solar_zenith_band_4": "solar_zenith",
 }
 
 MTL_PAIRS_RE = re.compile(r"(\w+)\s=\s(.*)")
@@ -258,7 +262,7 @@ def get_mtl_content(acquisition_path: Path, root_element=None) -> Tuple[Dict, st
                         mtl_doc, file_root_element = read_mtl(fp)
                         return mtl_doc, file_root_element, member.name
             else:
-                raise RuntimeError(f"MTL file not found in {str(acquisition_path)}")
+                raise RuntimeError(f"MTL file not found in {acquisition_path!s}")
 
     else:
         paths = list(acquisition_path.rglob("*_MTL.txt"))
@@ -296,7 +300,6 @@ def read_mtl(fp: Iterable[Union[str, bytes]], root_element=None) -> Tuple[Dict,
         lines: Iterable[Union[str, bytes]],
         key_transform: Callable[[str], str] = lambda s: s.lower(),
     ) -> dict:
-
         tree = {}
         for line in lines:
             # If line is bytes-like convert to str
@@ -315,7 +318,7 @@ def read_mtl(fp: Iterable[Union[str, bytes]], root_element=None) -> Tuple[Dict,
 
     tree = _parse_group(fp)
     if root_element is None:
-        root_element = list(tree.keys())[0]
+        root_element = next(iter(tree.keys()))
     return tree[root_element], root_element
 
 
@@ -490,8 +493,7 @@ def prepare_and_write(
                     usgs_file_type.startswith("band_")
                     and (
                         # The older collection called quality a "band"
-                        "quality"
-                        not in usgs_file_type
+                        "quality" not in usgs_file_type
                     )
                 ),
             )
