@@ -3,6 +3,7 @@ Prepare eo3 metadata for Sentinel-2 Level 1C data produced by Sinergise or ESA.
 
 Takes ESA zipped datasets or Sinergise dataset directories
 """
+
 import fnmatch
 import json
 import os.path
@@ -301,9 +302,7 @@ def prepare_and_write(
             )
 
         dataset_id, metadata_path = p.done(embed_location=embed_location)
-        doc = serialise.from_doc(
-            p.written_dataset_doc, skip_validation=True, normalise_properties=False
-        )
+        doc = serialise.from_doc(p.written_dataset_doc, skip_validation=True)
         if not doc.locations:
             doc.locations = [names.resolve_location(dataset_location)]
         return doc, metadata_path
@@ -765,12 +764,10 @@ def main(
             )
 
     def find_jobs() -> Iterable[Job]:
-
         region_lookup = RegionLookup()
 
         nonlocal input_relative_to, embed_location
         for input_path in datasets:
-
             first = True
             for found_dataset in find_inputs_in_path(input_path):
                 _LOG.debug("found_dataset", name=found_dataset.name)
@@ -902,8 +899,7 @@ def main(
 
                     if always_granule_id or (
                         # None means 'auto': ie. automatically include granule id when there are multiple granules
-                        always_granule_id is None
-                        and len(granule_offsets) > 1
+                        always_granule_id is None and len(granule_offsets) > 1
                     ):
                         yaml_filename = (
                             f"{found_dataset.name}.{granule_offset}.odc-metadata.yaml"
@@ -956,6 +952,10 @@ def main(
                             output_yaml_path=job.output_yaml_path,
                         )
                     else:
+                        file_already_existed = (
+                            overwrite_existing and job.output_yaml_path.exists()
+                        )
+
                         dataset, path = prepare_and_write(
                             job.dataset_path,
                             job.output_yaml_path,
@@ -964,9 +964,26 @@ def main(
                             embed_location=job.embed_location,
                         )
                         _LOG.info(
-                            "Wrote dataset", dataset_id=dataset.id, datsaset_path=path
+                            "Wrote dataset",
+                            dataset_id=dataset.id,
+                            dataset_path=path,
+                            output_yaml=job.output_yaml_path,
                         )
-                        on_success(dataset, path)
+
+                        try:
+                            on_success(dataset, path)
+                        except Exception:
+                            # If the post-processing function fails, we don't want this "unfinished" file still around.
+                            # We'd rather let it be re-created next time.
+                            if not file_already_existed:
+                                _LOG.info(
+                                    "Cleaning new file due to indexing failure.",
+                                    output_yaml=job.output_yaml_path,
+                                )
+                                job.output_yaml_path.unlink()
+
+                            raise
+
                     successes += 1
                 except Exception:
                     _LOG.exception("failed_job", job=job)
@@ -1015,7 +1032,7 @@ def _write_dataset_safe(job: Job) -> Union[Tuple[DatasetDoc, Path], str]:
     A wrapper around `prepare_and_write` that catches exceptions and makes them
     serialisable as error strings.
 
-    (for use in multiprocessing pools etc)
+    (for use in multiprocessing pools etc.)
     """
     try:
         dataset, path = prepare_and_write(
