@@ -7,6 +7,7 @@ They arrive as a *.tar.gz with inner uncompressed tiffs, which Josh's tests have
 We compress the inner tiffs and store them in an uncompressed tar. This allows random reads within the files.
 We also append a checksum file at the end of the tar.
 """
+
 import copy
 import io
 import socket
@@ -318,7 +319,7 @@ def _recompress_image(
     if len(input_image.indexes) != 1:
         raise ValueError(
             f"Expecting one-band-per-tif input (USGS packages). "
-            f"Input has multiple layers {repr(input_image.indexes)}"
+            f"Input has multiple layers {input_image.indexes!r}"
         )
 
     array: numpy.ndarray = input_image.read(1)
@@ -415,7 +416,7 @@ def main(
                 )
             else:
                 raise ValueError(
-                    f"Expected either tar.gz or a dataset folder. " f"Got: {repr(path)}"
+                    f"Expected either tar.gz or a dataset folder. " f"Got: {path!r}"
                 )
 
             if not success:
