@@ -25,6 +25,8 @@ from shapely.geometry.base import BaseGeometry
 from eodatasets3.model import ODC_DATASET_SCHEMA_URL, DatasetDoc, Eo3Dict
 from eodatasets3.properties import FileFormat
 
+converter = cattr.Converter()
+
 
 def _format_representer(dumper, data: FileFormat):
     return dumper.represent_scalar("tag:yaml.org,2002:str", f"{data.name}")
@@ -196,9 +198,7 @@ METADATA_TYPE_SCHEMA = _load_schema_validator(
 )
 
 
-def from_doc(
-    doc: Dict, skip_validation=False, normalise_properties=False
-) -> DatasetDoc:
+def from_doc(doc: Dict, skip_validation=False) -> DatasetDoc:
     """
     Parse a dictionary into an EO3 dataset.
 
@@ -209,29 +209,22 @@ def from_doc(
     :param skip_validation: Optionally disable validation (it's faster, but I hope your
             doc is structured correctly)
     """
-
+    doc = doc.copy()
     if not skip_validation:
+        # don't error if properties 'extent' or 'grid_spatial' are present
+        if doc.get("extent"):
+            del doc["extent"]
+        if doc.get("grid_spatial"):
+            del doc["grid_spatial"]
         DATASET_SCHEMA.validate(doc)
 
     # TODO: stable cattrs (<1.0) balks at the $schema variable.
-    doc = doc.copy()
     del doc["$schema"]
     location = doc.pop("location", None)
     if location:
         doc["locations"] = [location]
 
-    c = cattr.Converter()
-    c.register_structure_hook(uuid.UUID, _structure_as_uuid)
-    c.register_structure_hook(BaseGeometry, _structure_as_shape)
-    c.register_structure_hook(
-        Eo3Dict,
-        partial(_structure_as_stac_props, normalise_properties=normalise_properties),
-    )
-
-    c.register_structure_hook(Affine, _structure_as_affine)
-
-    c.register_unstructure_hook(Eo3Dict, _unstructure_as_stac_props)
-    return c.structure(doc, DatasetDoc)
+    return converter.structure(doc, DatasetDoc)
 
 
 def _structure_as_uuid(d, t):
@@ -274,6 +267,16 @@ def _structure_as_shape(d, t):
     return shape(d)
 
 
+converter.register_structure_hook(uuid.UUID, _structure_as_uuid)
+converter.register_structure_hook(BaseGeometry, _structure_as_shape)
+converter.register_structure_hook(
+    Eo3Dict,
+    partial(_structure_as_stac_props, normalise_properties=False),
+)
+converter.register_structure_hook(Affine, _structure_as_affine)
+converter.register_unstructure_hook(Eo3Dict, _unstructure_as_stac_props)
+
+
 def to_doc(d: DatasetDoc) -> Dict:
     """
     Serialise a DatasetDoc to a dict
@@ -458,8 +461,8 @@ class ClickDatetime(click.ParamType):
         except ValueError:
             self.fail(
                 (
-                    "Invalid date string {!r}. Expected any ISO date/time format "
-                    '(eg. "2017-04-03" or "2014-05-14 12:34")'.format(value)
+                    f"Invalid date string {value!r}. Expected any ISO date/time format "
+                    '(eg. "2017-04-03" or "2014-05-14 12:34")'
                 ),
                 param,
                 ctx,
