@@ -1,6 +1,7 @@
 """
 Validate ODC dataset documents
 """
+
 import collections
 import enum
 import math
@@ -543,7 +544,7 @@ def validate_product(doc: Dict) -> ValidationMessages:
                 if measurements_with_this_name:
                     seen_in = " and ".join(
                         repr(s)
-                        for s in ([measurement_name] + measurements_with_this_name)
+                        for s in ([measurement_name, *measurements_with_this_name])
                     )
 
                     # If the same name is used by different measurements, its a hard error.
@@ -909,7 +910,6 @@ def _match_product(
 
     # We we have nothing, give up!
     if (not matching_products) and (not product):
-
         # Find the product that most closely matches it, to helpfully show the differences!
         closest_product_name = None
         closest_differences = None
@@ -957,11 +957,13 @@ def _match_product(
             _error(
                 "product_match_clash",
                 "Multiple products match the given dataset",
-                hint=f"Maybe you need more fields in the 'metadata' section?\n"
-                f"Claims to be a {specified_product_name!r}, and matches {matching_names!r}"
-                if specified_product_name
-                else f"Maybe you need more fields in the 'metadata' section?\n"
-                f"Matches {matching_names!r}",
+                hint=(
+                    f"Maybe you need more fields in the 'metadata' section?\n"
+                    f"Claims to be a {specified_product_name!r}, and matches {matching_names!r}"
+                    if specified_product_name
+                    else f"Maybe you need more fields in the 'metadata' section?\n"
+                    f"Matches {matching_names!r}"
+                ),
             )
         )
         # (We wont pick one from the bunch here. Maybe they already matched one above to use in continuing validation.)
@@ -1292,7 +1294,6 @@ def _load_remote_product_definitions(
     from_datacube: bool = False,
     from_explorer_url: Optional[str] = None,
 ) -> Dict[str, Dict]:
-
     product_definitions = {}
     # Load any remote products that were asked for.
     if from_explorer_url:
