@@ -3,7 +3,6 @@ import hashlib
 import logging
 import os
 import typing
-from distutils import spawn
 from pathlib import Path
 from urllib.parse import urlparse
 
@@ -35,11 +34,14 @@ def find_exe(name: str):
     :return: the absolute path to the executable.
     :rtype: str
     """
-    executable = spawn.find_executable(name)
-    if not executable:
-        raise Exception(f"No {name!r} command found.")
+    from shutil import which
 
-    return executable
+    executable = which(name)
+
+    if executable is None:
+        raise Exception(f"No {name!r} command found.")
+    else:
+        return executable
 
 
 def calculate_file_sha1(filename):
@@ -187,9 +189,7 @@ class PackageChecksum:
         with output_file.open("wb") as f:
             f.writelines(
                 (
-                    "{}\t{}\n".format(
-                        str(hash_), str(filename.relative_to(output_file.parent))
-                    ).encode("utf-8")
+                    f"{hash_!s}\t{filename.relative_to(output_file.parent)!s}\n".encode()
                     for filename, hash_ in sorted(self._file_hashes.items())
                 )
             )
