@@ -5,17 +5,23 @@ This converts the HDF5 file (and sibling fmask/gqa files) into
 GeoTIFFS (COGs) with datacube metadata using the DEA naming conventions
 for files.
 """
+
 import contextlib
 import os
 import re
 import sys
+import zipfile
 from datetime import datetime, timedelta
 from enum import Enum
+from math import isnan
+from os.path import join
 from pathlib import Path
 from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple
 from uuid import UUID
 
 import attr
+import defusedxml.ElementTree as Et
+import fiona
 import numpy
 import rasterio
 from affine import Affine
@@ -58,12 +64,34 @@ FILENAME_TIF_BAND = re.compile(
 )
 PRODUCT_SUITE_FROM_GRANULE = re.compile("(L1[GTPCS]{1,2})")
 
+# Map wagl dataset band IDs to ESA MTD.xml Band IDs
+WAGL_TO_ESA_BAND_NUMBER = {
+    "1": "0",
+    "2": "1",
+    "3": "2",
+    "4": "3",
+    "5": "4",
+    "6": "5",
+    "7": "6",
+    "8": "7",
+    "8A": "8",
+    "9": "9",
+    "10": "10",
+    "11": "11",
+    "12": "12",
+}
+
 
 class ProductMaturity(Enum):
     provisional = "provisional"
     stable = "stable"
 
 
+# a dictionary of band_id: (mask_type, mask_uri)
+#     (the mask URI is a string loadable by fiona. Could be a URL or file path.)
+BandMasks = Dict[str, Tuple[str, str]]
+
+
 def _find_h5_paths(h5_obj: h5py.Group, dataset_class: str = "") -> List[str]:
     """
     Find all objects in a h5 of the given class, returning their path.
@@ -81,7 +109,10 @@ def _find_h5_paths(h5_obj: h5py.Group, dataset_class: str = "") -> List[str]:
 
 
 def _unpack_products(
-    p: DatasetAssembler, product_list: Iterable[str], h5group: h5py.Group
+    p: DatasetAssembler,
+    product_list: Iterable[str],
+    h5group: h5py.Group,
+    granule: "Granule",
 ) -> None:
     """
     Unpack and package the NBAR and NBART products.
@@ -92,14 +123,15 @@ def _unpack_products(
     for product in product_list:
         with sub_product(product, p):
             for pathname in [p for p in img_paths if f"/{product.upper()}/" in p]:
-
                 with do(f"Path {pathname!r}"):
                     dataset = h5group[pathname]
+                    band_masks = get_quality_masks(dataset, granule)
                     band_name = utils.normalise_band_name(dataset.attrs["alias"])
                     write_measurement_h5(
                         p,
                         f"{product}:{band_name}",
                         dataset,
+                        band_masks=band_masks,
                         overview_resampling=Resampling.average,
                         file_id=_file_id(dataset),
                     )
@@ -121,10 +153,183 @@ def _unpack_products(
                     )
 
 
+def get_quality_masks(
+    dataset: h5py.Dataset, granule: "Granule", strict=True
+) -> BandMasks:
+    """
+    Get a dictionary of any quality masks to apply for each band (if any).
+    """
+
+    if dataset.attrs["band_id"] not in WAGL_TO_ESA_BAND_NUMBER.keys():
+        return {}
+
+    platform_name = granule.wagl_metadata["source_datasets"]["platform_id"].lower()
+    if "sentinel" not in platform_name:
+        return {}
+
+    level1_data_path = granule.source_level1_data
+
+    granule_metadata_xml = Path(
+        granule.source_level1_metadata.accessories["metadata:s2_tile"].path
+    )
+
+    if not level1_data_path.exists():
+        raise ValueError(f"Input level 1 data not found {level1_data_path}")
+
+    if level1_data_path.suffix == ".zip":
+        product_root = granule_metadata_xml.parts[0]
+        # Parse tile metadata xml file
+        with zipfile.ZipFile(level1_data_path, "r") as z:
+            mtd_dict = raw_mask_offsets_from_granule_xml(
+                z.read(granule_metadata_xml.as_posix())
+            )
+            masks: BandMasks = {}
+            # Convert the raw mask offsets to loadable fiona paths.
+            for band_id in list(mtd_dict.keys()):
+                type_, location = mtd_dict[band_id]
+                mask_offset = join(product_root, location)
+                if mask_offset not in z.namelist():
+                    mask_offset = str(
+                        granule_metadata_xml.parent / "QI_DATA" / location
+                    )
+                info = z.getinfo(mask_offset)
+                # Mask under 500 bytes are empty, which kills fiona on load.
+                # (From scanning, non-empty ones in our collection are all far larger than 1000bytes,
+                #  so this doesn't need precision)
+                if info.file_size < 500:
+                    continue
+                masks[band_id] = (
+                    type_,
+                    f"zip+file://{level1_data_path.as_posix()}!/{mask_offset}",
+                )
+        return masks
+    # Sinergise data comes as a directory.
+    elif level1_data_path.is_dir():
+        mtd_dict = raw_mask_offsets_from_granule_xml(
+            (level1_data_path / granule_metadata_xml).read_bytes()
+        )
+        masks: BandMasks = {}
+        for band_id in list(mtd_dict.keys()):
+            type_, location = mtd_dict[band_id]
+            mask_path = level1_data_path / location
+            # Sinergise use the original ESA metadata document, but have a new directory structure
+            # So the metadata mask locations are (always?) wrong.
+            # (It's in a 'qi' folder with the original filename)
+            if not mask_path.exists():
+                mask_path = level1_data_path / "qi" / Path(location).name
+            if not mask_path.exists():
+                # Sinergise seems to simplify the mask filename.
+                mask_path = (
+                    level1_data_path / "qi" / Path(location).name.replace("MSK_", "")
+                )
+            if not mask_path.exists():
+                if strict:
+                    raise ValueError(
+                        f"Mask path {mask_path} does not exist for band {band_id}"
+                    )
+                else:
+                    # Nothing found, and we're not being strict.
+                    return {}
+
+            # Skip small ones. See reasoning in ESA block above.
+            if mask_path.stat().st_size < 500:
+                continue
+            masks[band_id] = (type_, mask_path.as_posix())
+        return masks
+    else:
+        raise RuntimeError(
+            "Input level 1 data must be a zip file (ESA) or directory (Sinergise)"
+        )
+
+
+def raw_mask_offsets_from_granule_xml(granule_metadata_xml: bytes):
+    root = Et.fromstring(granule_metadata_xml, forbid_dtd=True)
+
+    return {
+        # Append metadata bandId as key and mask type and path as tuple vals to dict
+        e.attrib["bandId"]: (e.attrib["type"], e.text)
+        for e in root.findall(".//Pixel_Level_QI/MASK_FILENAME")
+        if "bandId" in e.attrib
+        and "type" in e.attrib
+        and e.attrib["type"] in ("MSK_QUALIT", "MSK_TECQUA")
+    }
+
+
+def load_and_mask_data(g: h5py.Dataset, masks: BandMasks):
+    band_id = g.attrs["band_id"]
+    esa_band = WAGL_TO_ESA_BAND_NUMBER[band_id]
+    if esa_band not in masks:
+        # No mask needs to be applied.
+        data_array = g[:] if hasattr(g, "chunks") else g
+        return data_array
+
+    type_, mask_string_path = masks[esa_band]
+    if type_ == "MSK_TECQUA":
+        return mask_h5_vector(
+            g,
+            mask_string_path,
+        )
+    elif type_ == "MSK_QUALIT":
+        return mask_h5_raster(
+            g,
+            mask_string_path,
+        )
+    else:
+        raise ValueError(f"unknown mask type {type_}")
+
+
+def mask_h5_vector(
+    dataset: h5py.Dataset,
+    mask_string_path,
+):
+    """
+    Mask a hdf5 dataset using the gml files provided in the path.
+    """
+    data_array = dataset[:] if hasattr(dataset, "chunks") else dataset
+    # Open the gml file
+    with fiona.open(mask_string_path) as gml:
+        shapes = [feature["geometry"] for feature in gml]
+        if not shapes:
+            return data_array
+        mask_array = rasterio.features.rasterize(
+            shapes,
+            out_shape=data_array.shape,
+            fill=0,
+            transform=Affine.from_gdal(*dataset.attrs["geotransform"]),
+        )
+        return numpy.where(mask_array == 0, data_array, dataset.attrs["no_data_value"])
+
+
+def mask_h5_raster(
+    dataset: h5py.Dataset,
+    mask_string_path,
+):
+    """
+    Mask a hdf5 dataset using the gml files provided in the path.
+    """
+    data_array = dataset[:] if hasattr(dataset, "chunks") else dataset
+
+    # Open the .jp2 file
+    with rasterio.open(mask_string_path) as source_ds:
+        # Mask layer info:
+        # https://sentinel.esa.int/web/sentinel/technical-guides/sentinel-2-msi/level-1c/masks
+        # MSI lost data layer
+        source_layer1 = source_ds.read(3)
+        # MSI degraded data layer
+        source_layer2 = source_ds.read(4)
+        # Mask where missing OR degraded packets
+        source_layer = source_layer1 | source_layer2
+        masked_data = numpy.where(
+            source_layer == 0, data_array, dataset.attrs["no_data_value"]
+        )
+        return masked_data
+
+
 def write_measurement_h5(
     p: DatasetAssembler,
     full_name: str,
     g: h5py.Dataset,
+    band_masks: Optional[BandMasks] = None,
     overviews=images.DEFAULT_OVERVIEWS,
     overview_resampling=Resampling.nearest,
     expand_valid_data=True,
@@ -133,10 +338,10 @@ def write_measurement_h5(
     """
     Write a measurement by copying it from a hdf5 dataset.
     """
-    if hasattr(g, "chunks"):
-        data = g[:]
+    if band_masks:
+        data = load_and_mask_data(g, band_masks)
     else:
-        data = g
+        data = g[:] if hasattr(g, "chunks") else g
 
     product_name, band_name = full_name.split(":")
     p.write_measurement_numpy(
@@ -225,7 +430,6 @@ def get_oa_resolution_group(
     platform: str,
     oa_resolution: Optional[Tuple[float, float]],
 ) -> h5py.Group:
-
     # None specified? Figure out a default.
 
     if oa_resolution is None:
@@ -265,29 +469,39 @@ def _create_contiguity(
     """
     for product in product_list:
         contiguity = None
+        # A contiguity layer for each product
         for grid, band_name, path in p.iter_measurement_paths():
             if not band_name.startswith(f"{product.lower()}:"):
                 continue
             # Only our given res group (no pan band in Landsat)
             if grid.resolution_yx != resolution_yx:
                 continue
-
             with rasterio.open(path) as ds:
                 ds: DatasetReader
-                if contiguity is None:
-                    contiguity = numpy.ones((ds.height, ds.width), dtype="uint8")
-                    geobox = GridSpec.from_rio(ds)
-                elif ds.shape != contiguity.shape:
-                    raise NotImplementedError(
-                        "Contiguity from measurements of different shape"
-                    )
-
-                for band in ds.indexes:
-                    contiguity &= ds.read(band) > 0
+                out_shape = ds.shape
+                contiguity = numpy.ones(out_shape, dtype="uint8")
+                geobox = GridSpec.from_rio(ds)
+                break
 
         if contiguity is None:
-            secho(f"No images found for requested product {product}", fg="red")
-            continue
+            raise ValueError(f"no matching band with resolution {resolution_yx}")
+
+        for grid, band_name, path in p.iter_measurement_paths():
+            if not band_name.startswith(f"{product.lower()}:"):
+                continue
+            # Only our given res group (no pan band in Landsat)
+            if (p.platform.startswith("landsat")) and (
+                grid.resolution_yx != resolution_yx
+            ):
+                continue
+            with rasterio.open(path) as ds:
+                ds: DatasetReader
+                contiguity &= (
+                    ds.read(
+                        ds.count, out_shape=out_shape, resampling=Resampling.nearest
+                    )
+                    > 0
+                )
 
         p.write_measurement_numpy(
             f"oa:{product.lower()}_contiguity",
@@ -390,6 +604,7 @@ class Granule:
     wagl_hdf5: Path
     wagl_metadata: Dict
     source_level1_metadata: Optional[DatasetDoc]
+    source_level1_data: Optional[Path] = None
 
     fmask_doc: Optional[Dict] = None
     fmask_image: Optional[Path] = None
@@ -405,6 +620,7 @@ class Granule:
         wagl_hdf5: Path,
         granule_names: Optional[Sequence[str]] = None,
         level1_metadata_path: Optional[Path] = None,
+        level1_data_path: Optional[Path] = None,
         fmask_image_path: Optional[Path] = None,
         fmask_doc_path: Optional[Path] = None,
         s2cloudless_prob_path: Optional[Path] = None,
@@ -442,8 +658,13 @@ class Granule:
 
                 [wagl_doc] = loads_yaml(wagl_doc_field[()])
 
-                level1 = _load_level1_doc(
-                    wagl_doc, level1_metadata_path, allow_missing_provenance
+                # If they don't specify a level1, default to the one in the wagl doc.
+                if not level1_data_path:
+                    level1_data_path = Path(
+                        get_path(wagl_doc, ("source_datasets", "source_level1"))
+                    )
+                level1_doc = _load_level1_doc(
+                    level1_data_path, level1_metadata_path, allow_missing_provenance
                 )
 
                 fmask_image_path = fmask_image_path or wagl_hdf5.with_name(
@@ -459,6 +680,31 @@ class Granule:
                     [fmask_doc] = loads_yaml(fl)
 
                 if "sentinel" in wagl_doc["source_datasets"]["platform_id"].lower():
+                    # We need the level1 data for S2 in order to apply quality masks.
+                    if not level1_data_path.exists():
+                        # TODO: We could infer this from the 'location:' field of the metadata, when they
+                        #       have embedded locations
+                        # If the data directory is a sibling, we can infer it.
+                        match = METADATA_DOC_STEM.match(level1_metadata_path.name)
+                        if not match:
+                            raise ValueError(
+                                f"Can't infer level1 data path from {level1_data_path}"
+                            )
+
+                        # For ESA: It may be at an identical path, with zip suffix instead of *.odc-metadata.yaml
+                        level1_data_path = level1_metadata_path.parent / (
+                            match.group("stem") + ".zip"
+                        )
+                        if not level1_data_path.exists():
+                            # Or it may be a sinergise directory, with no suffix.
+                            level1_data_path = (
+                                level1_metadata_path.parent / match.group("stem")
+                            )
+                            if not level1_data_path.exists():
+                                raise ValueError(
+                                    f"Can't find level1 data for {level1_data_path}"
+                                )
+
                     s2cloudless_prob_path = (
                         s2cloudless_prob_path
                         or wagl_hdf5.with_name(f"{granule_name}.prob.s2cloudless.tif")
@@ -508,6 +754,8 @@ class Granule:
                         )
                 else:
                     tesp_doc_path = wagl_hdf5.with_name(f"{granule_name}.tesp.yaml")
+
+                tesp_doc = None
                 if tesp_doc_path.exists():
                     with tesp_doc_path.open("r") as fl:
                         [tesp_doc] = loads_yaml(fl)
@@ -516,7 +764,8 @@ class Granule:
                     name=granule_name,
                     wagl_hdf5=wagl_hdf5,
                     wagl_metadata=wagl_doc,
-                    source_level1_metadata=level1,
+                    source_level1_metadata=level1_doc,
+                    source_level1_data=level1_data_path,
                     fmask_doc=fmask_doc,
                     fmask_image=fmask_image_path,
                     s2cloudless_prob=s2cloudless_prob_path,
@@ -527,12 +776,17 @@ class Granule:
                 )
 
 
+# Get just  the stem of metadata documents (named 'docname.yaml' or 'docname.odc-metadata.yaml')
+METADATA_DOC_STEM = re.compile(
+    r"^(?P<stem>.+?)(?:\.odc-metadata)?\.yaml$", re.IGNORECASE
+)
+
+
 def _load_level1_doc(
-    wagl_doc: Dict,
+    level1_data_path: Path,
     user_specified_l1_path: Optional[Path] = None,
     allow_missing_provenance=False,
-):
-
+) -> DatasetDoc:
     if user_specified_l1_path:
         if not user_specified_l1_path.exists():
             raise ValueError(
@@ -540,7 +794,8 @@ def _load_level1_doc(
             )
         level1_path = user_specified_l1_path
     else:
-        level1_path = Path(get_path(wagl_doc, ("source_datasets", "source_level1")))
+        # We try to infer it from the data path
+        level1_path = level1_data_path
 
     # If a directory, assume "<dirname>.odc-metadata.yaml"
     if level1_path.is_dir():
@@ -631,9 +886,11 @@ def package(
             out_directory.absolute(),
             # WAGL stamps a good, random ID already.
             dataset_id=granule.wagl_metadata.get("id"),
-            naming_conventions="dea_s2"
-            if ("sentinel" in wagl_doc["source_datasets"]["platform_id"].lower())
-            else "dea",
+            naming_conventions=(
+                "dea_s2"
+                if ("sentinel" in wagl_doc["source_datasets"]["platform_id"].lower())
+                else "dea"
+            ),
         ) as p:
             _apply_wagl_metadata(p, wagl_doc)
 
@@ -685,7 +942,7 @@ def package(
             if granule.tesp_doc:
                 _take_software_versions(p, granule.tesp_doc)
 
-            _unpack_products(p, included_products, granule_group)
+            _unpack_products(p, included_products, granule_group, granule)
 
             if include_oa:
                 with sub_product("oa", p):
@@ -794,7 +1051,7 @@ def _read_gqa_doc(p: DatasetAssembler, doc: Dict):
 def _read_fmask_doc(p: DatasetAssembler, doc: Dict):
     for name, value in doc["percent_class_distribution"].items():
         # From Josh: fmask cloud cover trumps the L1 cloud cover.
-        if name == "cloud":
+        if name == "cloud" and not isnan(value):
             if "eo:cloud_cover" in p.properties:
                 del p.properties["eo:cloud_cover"]
             p.properties["eo:cloud_cover"] = value
@@ -853,17 +1110,21 @@ def _read_wagl_metadata(granule_group: h5py.Group):
 
     for i, path in enumerate(ancil_paths, start=2):
         wagl_doc.setdefault(f"wagl_{i}", {}).update(
-            list(loads_yaml(granule_group[path][()]))[0]["ancillary"]
+            next(iter(loads_yaml(granule_group[path][()])))["ancillary"]
         )
     return wagl_doc
 
 
 def _apply_wagl_metadata(p: DatasetAssembler, wagl_doc: Dict):
     source = wagl_doc["source_datasets"]
+
     p.datetime = source["acquisition_datetime"]
     p.platform = source["platform_id"]
     p.instrument = source["sensor_id"]
 
+    if "final_ancillaries" in wagl_doc:
+        p.properties["dea:final_ancillaries"] = wagl_doc["final_ancillaries"]
+
     try:
         p.processed = get_path(wagl_doc, ("system_information", "time_processed"))
     except PathAccessError:
